# Generated by Django 4.2.20 on 2025-06-23 10:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0007_message_imap_uid'),
        ('unibot', '0002_tool_bot_tools'),
    ]

    operations = [
        migrations.CreateModel(
            name='EncryptedCredential',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255)),
                ('encrypted_value', models.BinaryField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='credentials', to='unicom.account')),
            ],
            options={
                'indexes': [models.Index(fields=['account', 'key'], name='unibot_encr_account_4d9e0a_idx')],
                'unique_together': {('account', 'key')},
            },
        ),
    ]
