# Generated by Django 4.2.20 on 2025-10-31 09:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0017_add_chat_metadata'),
        ('unicrm', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='communicationmessage',
            unique_together={('communication', 'contact')},
        ),
        migrations.AddField(
            model_name='communication',
            name='channel',
            field=models.ForeignKey(blank=True, help_text='Channel used to send this communication.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='unicrm_communications', to='unicom.channel', verbose_name='Channel'),
        ),
        migrations.AddField(
            model_name='communication',
            name='subject_template',
            field=models.CharField(blank=True, help_text='Optional Jinja2 template used for the email subject line.', max_length=255, verbose_name='Subject template'),
        ),
        migrations.AddField(
            model_name='communicationmessage',
            name='draft',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='unicrm_links', to='unicom.draftmessage'),
        ),
        migrations.AlterField(
            model_name='communicationmessage',
            name='message',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='unicrm_links', to='unicom.message'),
        ),
    ]
