# Generated by Django 4.2.20 on 2025-11-03 12:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0017_add_chat_metadata'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='bounce_details',
            field=models.JSONField(blank=True, default=dict, help_text='Raw provider payload for the bounce event'),
        ),
        migrations.AddField(
            model_name='message',
            name='bounce_reason',
            field=models.TextField(blank=True, help_text='Provider-supplied reason for the bounce'),
        ),
        migrations.AddField(
            model_name='message',
            name='bounce_type',
            field=models.CharField(blank=True, choices=[('hard', 'Hard bounce'), ('soft', 'Soft bounce')], help_text='Classification of the bounce when available', max_length=20),
        ),
        migrations.AddField(
            model_name='message',
            name='bounced',
            field=models.BooleanField(default=False, help_text='Whether the email was reported as bounced by the provider'),
        ),
        migrations.AddField(
            model_name='message',
            name='time_bounced',
            field=models.DateTimeField(blank=True, help_text='When the bounce notification was received', null=True),
        ),
    ]
