# Generated by Django 4.2.20 on 2025-06-12 09:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('unicom', '0005_emailinlineimage_hash_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bot',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('category', models.CharField(max_length=255)),
                ('code', models.TextField()),
                ('request_category', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='unicom.requestcategory')),
            ],
        ),
    ]
