# Generated by Django 4.2.20 on 2025-06-07 05:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0003_alter_emailinlineimage_email_message'),
    ]

    operations = [
        migrations.CreateModel(
            name='MessageTemplateInlineImage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='message_template_inline_images/')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('content_id', models.CharField(blank=True, help_text='Content-ID for cid: references in HTML', max_length=255, null=True)),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='inline_images', to='unicom.messagetemplate')),
            ],
        ),
    ]
