# Generated by Django 4.2.20 on 2025-06-17 17:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unibot', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tool',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True)),
                ('code', models.TextField(blank=True, help_text='Python implementation including a top-level `tool_definition` dict')),
            ],
        ),
        migrations.AddField(
            model_name='bot',
            name='tools',
            field=models.ManyToManyField(blank=True, related_name='bots', to='unibot.tool'),
        ),
    ]
