# Generated by Django 4.2.20 on 2025-10-23 11:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0016_callbackexecution_tool_call'),
    ]

    operations = [
        migrations.AddField(
            model_name='chat',
            name='metadata',
            field=models.JSONField(blank=True, default=dict, help_text='Custom properties for filtering (e.g., project_id, department_id)'),
        ),
        migrations.AlterField(
            model_name='account',
            name='platform',
            field=models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email'), ('WebChat', 'WebChat')], max_length=100),
        ),
        migrations.AlterField(
            model_name='channel',
            name='platform',
            field=models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email'), ('WebChat', 'WebChat')], max_length=100),
        ),
        migrations.AlterField(
            model_name='chat',
            name='platform',
            field=models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email'), ('WebChat', 'WebChat')], max_length=100),
        ),
        migrations.AlterField(
            model_name='message',
            name='platform',
            field=models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email'), ('WebChat', 'WebChat')], max_length=100),
        ),
        migrations.AlterField(
            model_name='update',
            name='platform',
            field=models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email'), ('WebChat', 'WebChat')], max_length=100),
        ),
    ]
