# Generated by Django 4.2.20 on 2025-11-09 09:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicrm', '0010_communication_evergreen_refreshed_at'),
    ]

    operations = [
        migrations.AlterField(
            model_name='company',
            name='domain',
            field=models.CharField(blank=True, help_text='Primary email domain for this company (optional)', max_length=255, unique=True, verbose_name='Email domain'),
        ),
        migrations.AlterField(
            model_name='company',
            name='name',
            field=models.CharField(max_length=255, unique=True, verbose_name='Name'),
        ),
    ]
