# Generated by Django 4.2.20 on 2025-11-05 20:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('unicrm', '0008_rename_unicrm_msg_status_sched_idx_unicrm_comm_status_2f3c8e_idx_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='communicationmessage',
            name='draft',
        ),
    ]
