# Generated by Django 4.2.20 on 2025-06-23 12:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unibot', '0004_credentialfielddefinition_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='credentialfielddefinition',
            name='key',
            field=models.CharField(help_text="Globally unique identifier for this credential (e.g., 'GOOGLE_PLACES_API_KEY', 'STRIPE_SECRET_KEY_LIVE')", max_length=255, unique=True),
        ),
    ]
