# Generated by Django 4.2.20 on 2025-11-03 12:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicrm', '0005_contact_user_alter_communication_status_summary'),
    ]

    operations = [
        migrations.AddField(
            model_name='contact',
            name='email_bounce_type',
            field=models.CharField(blank=True, help_text='Classification of the most recent bounce (e.g. hard or soft).', max_length=20, verbose_name='Email bounce type'),
        ),
        migrations.AddField(
            model_name='contact',
            name='email_bounced',
            field=models.BooleanField(default=False, help_text='True when a recent outbound email to this contact bounced.', verbose_name='Email bounced'),
        ),
        migrations.AddField(
            model_name='contact',
            name='email_bounced_at',
            field=models.DateTimeField(blank=True, help_text='Most recent bounce timestamp cached for this contact.', null=True, verbose_name='Email bounced at'),
        ),
    ]
