# Generated by Django 4.2.20 on 2025-08-21 21:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0008_add_all_members_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='media_type',
            field=models.CharField(choices=[('text', 'Text'), ('html', 'HTML'), ('image', 'Image'), ('audio', 'Audio'), ('tool_call', 'Tool Call'), ('tool_response', 'Tool Response')], default='text', max_length=15),
        ),
    ]
