# Generated by Django 4.2.24 on 2025-10-02 10:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0015_simplify_callback_execution'),
    ]

    operations = [
        migrations.AddField(
            model_name='callbackexecution',
            name='tool_call',
            field=models.ForeignKey(blank=True, help_text='Optional: The ToolCall that created this button (if from a tool)', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='button_callbacks', to='unicom.toolcall'),
        ),
    ]
