# Generated by Django 4.2.20 on 2025-11-10 05:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicrm', '0011_add_unique_constraints_to_company'),
    ]

    operations = [
        migrations.AddField(
            model_name='company',
            name='attributes',
            field=models.JSONField(blank=True, default=dict, help_text='Arbitrary data map available to template variables.', verbose_name='Custom attributes'),
        ),
        migrations.AddField(
            model_name='company',
            name='industry',
            field=models.CharField(blank=True, help_text='Primary industry or vertical focus for this company (optional).', max_length=255, verbose_name='Industry'),
        ),
    ]
