# Generated by Django 4.2.20 on 2025-11-17 09:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unicom', '0018_message_bounce_details_message_bounce_reason_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='draftmessage',
            name='skip_reacher_validation',
            field=models.BooleanField(default=False, help_text='Skip Reacher email validation when sending this draft', verbose_name='Skip Reacher validation'),
        ),
    ]
